;---------------------------------------------------------------------------;
; Allocate Memory                                                           ;
;                                                                           ;
; ENTRY: BX contains starting offset of buffer area.                        ;
;        (Note, this area contains initialization code and data which will  ;
;        be overlaid once we start reading into the buffers.)               ;
;                                                                           ;
;   Release memory used by initialization routine and allocate 2000h pages  ;
;   (128k) memory for use as file buffers. If this fails get as much as     ;
;   possible. Divide memory allocated into 2 buffers - 1 for each file.     ;
;   Leaves ES as segment pointer to first file segment (OLD file)           ;
;---------------------------------------------------------------------------;
Allocate_Memory:

; Now determine how many paragraphs (16 bytes) program plus stack needs:
        ADD     BX,15                     ;Round up to nearest PARA
        MOV     CL,4
        SHR     BX,CL                     ;Divide bytes of storage by 16

        MOV     AH,4Ah                    ;Dealloc all but needed (BX) paras
        INT     21h

        MOV     BX,2000h                  ;Request 128K for input buffers
        MOV     AH,48h                    ;DOS request mem function
        INT     21h
        JNC     Mem_OK                    ;If memory is available

        MOV     DX,offset Mem_loss_Msg    ;Display msg informing lack of mem
        MOV     AH,09H                    ;DOS display string function
        INT     21H
; Attempt to allocate what little memory is available and use that
        MOV     AX,BX                     ;Paragraphs of memory free
        SHR     AX,1                      ;Divide by two: Space for each file
        MOV     File_Size_PARAs,AX        ;Each file's size in pargraphs
        MOV     CL,4                      ;Prepare to shift left 4 bits (*16)
        SHL     AX,CL                     ;Mult by 16 = number of bytes per file
        MOV     File_Size_Bytes,AX        ;Space for each file in bytes
        MOV     AH,48h                    ;DOS alloc mem func:BX = para avail
        INT     21h                       ;Get what memory we can get
        JNC     Mem_OK                    ;If Alloc worked 2nd time around

; Fatal memory error:
        MOV     DX,offset Mem_ERR_Msg     ;Else: give message and give up
        MOV     AH,09H                    ;DOS display string function
        INT     21H
        CALL    Page_Wait                 ;Beep and force user to hit a key
        MOV     AX,4C14h                  ;   terminate with 20 error level
        INT     21h

Mem_OK:
        MOV     ES,AX                     ;ES points to start of allcoated block
        RET
