
;-----------------------------------------------------------------------
; MEMORY MANAGER
;-----------------------------------------------------------------------
; memory manager data area
;
packet  struc
  fwd	dw	?	;forward chain
  bak	dw	?	;backward chain
  adr	dw	?	;paragraph address
  siz	dw	?	;size in paragraphs
packet	ends

free_packet_chain  label	word
 first_free_packet packet <free_packet_buffer,last_free_packet,-1,0>
 free_packet_buffer	dw  900 dup (?,?,?,?)
 last_free_packet  packet <first_free_packet,(offset last_free_packet)-size packet,0,0>
free_memory_chain   label	word
        dummy1  packet <first1,first1,-1,0>	;dummy header
        first1	packet <dummy1,dummy1,0,0>	;first packet
allocated_memory_chain  label	word
        dummy2  packet <dummy2,dummy2,0,0>	;dummy header

;--------------
; memory manager initialization
;   inputs  -  AX = offset of first available memory location
;
memory_mgr_init:
	add	ax,15			;adjust for paragraph conversion
	mov	cl,4
	shr	ax,cl			;convert offset to paragraphs
	mov	cx,cs			;get our segment
	add	ax,cx			;compute starting segment
	mov	first1.adr,ax		;save starting address
;
; compute size of memory
;
	mov	bx,word ptr ds:[2]	;get total mem. avail. (paragraphs)
	sub	bx,ax			;compute (total_mem)-(our start adr)
	mov	first1.siz,bx		;save amount of mem. available.
;
; chain up the free packets
;
	mov	si,offset first_free_packet			;bak ptr
	mov	di,offset free_packet_buffer+(size packet)	;fwd ptr
	mov	bx,offset free_packet_buffer			;current pkt
free_lp:cmp	bx,offset last_free_packet			;check if done
	je	end_free_lp					;jump if done
	mov	[bx.bak],si
	mov	[bx.fwd],di
	add	bx,size packet
	add	si,size packet
	add	di,size packet
	jmp	free_lp
end_free_lp:
	ret
;-------------------------
; allocate memory
;  inputs  ax=size
;  output  es=paragraph  (if no carry)
;
allocate_memory:
	mov	bx,offset free_memory_chain	;get dummy packet
aloc1:	mov	bx,[bx.fwd]			;get next packeti
	cmp	bx,offset free_memory_chain	;check if at end of chain
	je	no_room				;jump if out of memory
	cmp	ax,[bx.siz]			;check if room here
	ja	aloc1				;jump if not enough mem.
;
; packet with enough memory has been found
;
	mov	es,[bx.adr]			;get memory address
	add	[bx.adr],ax			;move address fwd
	sub	[bx.siz],ax			;remove chunk of memory
	jnz	aloc2				;jump if some mem. left
	call	extract_packet			;all mem. used, remove pkt
	jmp	aloc3				;go reuse this pkt. in alloc. chain
aloc2:	mov	si,bx				;save -bx- (used by no_room1)
	mov	bx,offset free_packet_chain
	cmp	bx,[bx.fwd]			;check if more packets avail.
	je	no_room1			; and jump if out of packets
	mov	bx,[bx.fwd]
	call	extract_packet
aloc3:
	mov	[bx.adr],es			;put address in new packet
	mov	[bx.siz],ax			;put size in new packet
;
; bx - points at extracted blank packet
; ax - size of memory area allocated
; es - points at allocated memory area
;
; insert packet in the allocated_memory_chain
;
	mov	si,offset allocated_memory_chain
	mov	cx,es			;get memory segment
aloc5:	mov	si,[si.fwd]		;get next packet
	cmp	cx,[si.adr]		;check if insertion point
	jb	aloc5			;loop till point found
	call	insert_packet
	clc
	ret
no_room1:
	sub	[si.adr],ax		;restore packet
	add	[si.siz],ax		;  origional state
no_room:
	stc
	ret
;------------------------
; release memory
;  inputs ax=paragraph address
;  registers modified:   ax,bx,cx,si,di
;
release_memory:
	mov	bx,offset allocated_memory_chain
rel2:	mov	bx,[bx.fwd]		;get next packet
	cmp	bx,offset allocated_memory_chain
	je	bad_release		;jump if error
	cmp	ax,[bx.adr]		;check if release point
	jne	rel2			;loop till release point found
;
; bx - points at packet to release
;
	call	extract_packet
;
; scan free chain for insertion point
;
	mov	si,offset free_memory_chain
rel3:	mov	si,[si.fwd]		;get next free mem. pkt
	cmp	[si.adr],ax		;check for insertion point
	jb	rel3			;loop till insertion point found
;
; point to add memory back into free_memory_chain has been found.  Look for
; adjacent packets which can have this memory appended.
;
; bx - release packet
; si - insertion? point
; ax - address of released memory
;
	mov	di,[si.bak]		;get previous packet
	mov	cx,[di.adr]		;get previous adr
	cmp	cx,-1			;check if previous is dummy header
	je	rel4			; and jump if prev. is header
	add	cx,[di.siz]		;compute next seg
	cmp	ax,cx			;check if append to end possible
	jne	rel4			;jump in can't append
;
	mov	ax,[bx.siz]		;get size of released mem.
	add	[di.siz],ax		;append to old size
;
; existing allocated packet was updated, so this packet can be reused
;
	push	di
	mov	si,offset free_packet_chain
	call	insert_packet
	pop	di
;
; check if forward packet can now be combined?
;
	mov	ax,[di.adr]		;get current address
	add	ax,[di.siz]		;compute end of this block
	mov	si,[di.fwd]		;get ptr to forward packet
	cmp	ax,[si.adr]		;check if combine possible
	jne	rel7
;
; combine free_memory packets pointed at by si & di
;
	mov	ax,[si.siz]		;get size of fwd packet
	add	[di.siz],ax		;bump size of front packet
	mov	bx,si			;setup to remove packet
	call	extract_packet
	mov	si,offset free_packet_chain
	call	insert_packet
	jmp	rel7	
;
; check forward packet for possible append
;
rel4:	add	ax,[bx.siz]		;compute end of released mem.
	cmp	[si.adr],ax		;check if contigious
	jne	rel6			;jump if can't append
	mov	ax,[bx.adr]		;get released address
	mov	[si.adr],ax		;store new address
	mov	ax,[bx.siz]
	add	[si.siz],ax		;compute new size
;
; free up this packet
;
rel5:	mov	si,offset free_packet_chain
;
; could not append to existing free_memory packet.
;
rel6:	call	insert_packet
rel7:	clc
	ret
bad_release:
	stc
	ret
;------------------------
; extract packet from chain
;   inputs  bx=packet offset
;   output  bx=packet
;
extract_packet:
	mov	si,[bx.fwd]		;get fwd
	mov	di,[bx.bak]		;get bak
	mov	[di.fwd],si		;put new fwd ptr in bak packet
	mov	[si.bak],di		;put new bak ptr in fwd packet
	ret
;------------------------
; insert packet into chain
;    inputs  si=insertion point
;            bx=packet offset
;
insert_packet:
	mov	di,[si.bak]		;get bak packet
;
; fill in the new packet
;
	mov	[bx.fwd],si		;new fwd
	mov	[bx.bak],di		;new bak
;
	mov	[di.fwd],bx		;point bak packet at new
	mov	[si.bak],bx		;point fwd packet at new
	ret
;
