;
; This file contains routines to handle extended memory.  It will
; use raw extended memory or XMS memory.  If a 286 processor is
; present it will use the LOADALL transfer method,  but before
; using LOADALL it will test access and do a timing test to see
; if the BIOS has implemented a faster method.
;
; entry points are;  setup_ext_memory
;                    copy_from_program
;                    copy_to_program
;
;------------------------------------------------------------------------
;		     LOADALL read logic	database
;-------------------------------------------------------------------------
; read/write data area for extended memory transfers
;
desc  struc
  low_adr  dw  ?		;low address bits
  hi_adr   db  ?		;high	address	bits
  acc	   db  ?		;access rights byte
  limit	   dw  ?		;segment limit
desc  ends

	even
save_adr800_blk		DW	33H	DUP(0)

;------------------------------------------------
align 4

loadall_buf		DW	0,0,0		;unused
machine_status_word	DW	0
			dw	7	DUP (0)
tr_selector		dw	0
processor_flags		dw	0
processor_ip		DW	L01ED
ldtr_selector		dw	0
ds_selector		dw	0		; -ds- segment
ss_selector		dw	0		;stack segment
cs_selector		dw	0		; -cs- segment
es_selector		dw	0		; -es- segment
di_selector		dw	0
si_selector		dw	0
bp_selector		dw	0
sp_selector		dw	0
bx_selector		dw	0
dx_selector		dw	0
cx_selector		dw	0
ax_selector		dw	80h

es_descriptor		desc<0,0,92h,0ffffh>
cs_descriptor		desc<0,0,92h,0ffffh>
ss_descriptor		desc<0,0,92h,0ffffh>
ds_descriptor		desc<0,0,92h,0ffffh>

global_descriptor_table	db	0,0,0,0,0,0
local_descriptor_table	db	0,0D0H,0,0FFH,88H,0
interrupt_descriptor_table	desc<0,0,0,0>
tss_descriptor		db	0,0C0H,0,0FFH,0,08h

BPB_array		DB	37H,0
;-----------------------------------------------------------------------;
;	Segment Descriptor (part of Global Descriptor	Table)		 ;
;-----------------------------------------------------------------------;
DESC	STRUC			  ;data segment descriptor
 DESC_LMT   DW	0		 ;segment limit (length)
 DESC_BASEL DW	0		 ;bits 15-0 of physical address
 DESC_BASEH DB	0		 ;bits 23-16 of physical address
 DB	0			 ;access rights	byte
 DW	0			 ;reserved
DESC	ENDS

align 4
;-----------------------------------------------------------------------
; bios extended	memory database
;-----------------------------------------------------------------------;
;	Global Descriptor Table (GDT), used for extended memory moves	;
;-----------------------------------------------------------------------;
;Access	Rights Byte (93H) is
;	P=1	(segment is mapped into physical memory)
;	E=0	(data segment descriptor)
;	D=0	(grow up segment,	offsets	must be	<= limit)
;	W=1	(data segment may	be written into)
;	DPL=0	(privilege level 0)

GDT	LABEL	BYTE		 ;begin	global descriptor table
	DESC	<>		;dummy descriptor
	 DESC	<>		 ;descriptor	for GDT	itself
SRC	DESC	<512,,,93H,>	;source descriptor
TGT	DESC	<512,,,93H,>	;target descriptor
	 DESC	<>		 ;BIOS CS descriptor
	 DESC	<>		 ;stack segment descriptor
;-------------------------------------------------------------------------
; read block from xms memory
;  inputs: dx,ax = byte	address	(offset	from start of block)
;	   es:bx = destination
;
;	   xms_from_read_handle	- setup	by allocate_xms
;	   xms_read_len		- setup	by get_disk_info.
;
read_xms_packet	label	dword
  xms_read_len		dd	512	;length of read in bytes	(sector	size)
  xms_from_read_handle		dw	0	;from xms driver
  xms_from_read_address	dd	0	;byte address from	start of xms block
  xms_to_read_handle	dw	0	;always zero
  xms_to_read_offset	dw	0	;store buffer	offset here
  xms_to_read_segment	dw	0	;store buffer segment here

read_xms:
	shl	cx,1
	mov	word ptr xms_read_len,cx

	mov	xms_to_read_offset,bx	;save buffer	offset
	mov	xms_to_read_segment,es	;save buffer segment

	mov	word ptr [xms_from_read_address],ax
	mov	word ptr [xms_from_read_address+2],dx

	mov	si,offset read_xms_packet
	mov	ah,0bh
	call	xms_control
	cmp	ax,1
	jne	xms_failure
	ret

;--------------------------------------------------------------
; write	block to xms memory
;  inputs:  dx,ax = byte address (offset from start of block)
;	    es:bx = source data
;
write_xms_packet	label	dword
  xms_write_len		dd	512	;length	of write in bytes (sector size)
  xms_from_write_handle	dw	0	;always zero for conventional memory
  xms_from_write_offset	dw	0	;store buffer offset here
  xms_from_write_segment dw	0	;store buffer segment here
  xms_to_write_handle	dw	0	;handle from	xms driver
  xms_to_write_address	dd	0	;byte address from start of	xms block

write_xms:
	shl	cx,1
	mov	word ptr xms_write_len,cx

	mov	xms_from_write_offset,bx
	mov	xms_from_write_segment,es

	mov	word ptr [xms_to_write_address],ax
	mov	word ptr [xms_to_write_address+2],dx

	mov	si,offset write_xms_packet
	mov	ah,0bh
	call	xms_control
	cmp	ax,1
	jne	xms_failure
	ret
;-------------------------
xms_failure:
	jmp	ext_transfer_error


;------------------------------------------------------------------------------
; copy data from program to users/local buffer
;  inputs:    ax = from	address	(extended memory) 1meg base assumed
;	   es:di = to address (conventional memory)
;	      cx = words to move
;   output - ah	= error	code, or zero if no error
;
copy_from_program:
	mul	C256			;convert 256byte block address
	mov	bx,di			;move -to-	offset to bx
	cmp	extended_flag,3
	je	read_xms		;jmp if xms present
	add	dl,10h			;	 to extended byte address
	cmp	memory_type,6
	je	@extended_read		;use loadall if type 2
	mov	SRC.DESC_BASEL,ax	;store target
	mov	SRC.DESC_BASEH,dl	;  address

	mov	ax,es			;get users	buffer segment
	mul	C16			;convert to byte address
	add	ax,bx			;add offset
	adc	dl,0			;add any carry to high address byte
	mov	TGT.DESC_BASEL,AX	;store source
	mov	TGT.DESC_BASEH,dl	;  address
	jmp	short do_transfer
;------------------------------------------------------------------------------
; copy data to program (extended memory)
;   inputs:  es:si = from address (conventional	memory)
;		ax = to	address	from packet, adjusted by index to data
;		cx = words to move
;  outputs - ah	= error	code, or zero if no errors
;
;
copy_to_program:
	mul	C256			;convert 256byte block address
	mov	bx,si
	cmp	extended_flag,3
	je	write_xms		;jmp if xms present
	add	dl,10h			;convert to extended byte	address
	cmp	memory_type,6
	je	@extended_write
	mov	TGT.DESC_BASEL,ax	;store target
	mov	TGT.DESC_BASEH,dl	;  address

	mov	ax,es			;get users	buffer segment
	mul	C16			;convert to byte address
	add	ax,bx			;add offset
	adc	dl,0			;add any carry to high address byte
	mov	SRC.DESC_BASEL,AX	;store source
	mov	SRC.DESC_BASEH,dl	;  address

do_transfer:
	push	bp
	shl	cx,1			;convert -cx- to bytes
	mov	SRC.DESC_LMT,cx
	mov	TGT.DESC_LMT,cx
	mov	ax,cs			;point
	mov	es,ax			;	-es:si-
	mov	si,offset GDT		;	  at GDT
	shr	cx,1		    ;number of words	to transfer
	mov	ah,87h			;block move code
	int	15h			;call bios
	or	ah,ah
	jnz	ext_transfer_error	;jump if error occured
	pop	bp
	ret				;if -ah- is non	zero an	error occured
ext_transfer_error:
	mov	dx,offset error_code
	mov	ah,9
	int	21h
	sti				;enable	interrupts so reboot is	possible
forever: jmp	forever
;------------------------------------------------------------------------
;		     LOADALL read logic
;-------------------------------------------------------------------------
; Subroutine to	read one sector	or data	from extended memory
;   inputs - ax	= 256 byte block address of extended memory
;		  (1meg	assumed)
;	     es:bx = destination in our	conventional memory
;   output - ah	= error	code, or zero if no error
;
@extended_read:
	      MOV	word	ptr ds:ds_descriptor,AX
	MOV	byte ptr ds:ds_descriptor+2,DL
	MOV	di_selector,bx
	MOV	WORD PTR si_selector,0
	MOV	SI,OFFSET es_descriptor
	jmp	short @do_transfer
;-------------------------------------------------------------------------
; Subroutine to	write one sector of data to extended memory
;  inputs -  ax	= 256 byte block address of extended memory
;		  (1meg	assumed)
;	     es:bx = destination
;
@extended_write:
	MOV	word ptr ds:es_descriptor,AX
	MOV	byte ptr ds:es_descriptor+2,DL
	MOV	si_selector,bx
	MOV	WORD PTR di_selector,00
	MOV	SI,OFFSET ds_descriptor
;--------------------------------------------------------------------------
;
; common read/write setup
;
@do_transfer:
	push	bp
@L01A2:	MOV	AX,ES			;move buffer segment to -ax-
	CALL	seg_to_abs		;ax*16 -> ds:si
	MOV	AX,SS
	MOV	ss_selector,AX
	MOV	SI,OFFSET ss_descriptor
	CALL	seg_to_abs		;ax*16 -> ds:si
	MOV	sp_selector,SP
	MOV	AX,0DF90h
	push	cx
	CALL	command_8042		;ah=8042 command
	pop	cx
	JNZ	@ext_transfer_error
	MOV	WORD PTR cx_selector,cx
;
; save the contents of location	800h
;
	MOV	AX,0080h
	MOV	DS,AX
	PUSH	CS
	POP	ES
	XOR	SI,SI
	MOV	DI,OFFSET save_adr800_blk
	MOV	CX,033H
	CLD
	CLI
	REPZ	MOVSW		;80:0 -> cs:2b9  33h times
;
; move our data	to location 800h
;
	PUSH	DS
	PUSH	ES
	POP	DS
	POP	ES
	XOR	DI,DI
	MOV	SI,OFFSET loadall_buf
	MOV	CX,033H
	REPZ	MOVSW			;cs:31f -> 80:0  33h times
;
; the shadow registers can now be setup	to do the transfer
;
	DB	0FH,05			;load all 286 registers from loc. 800h
L01ED:	REP	MOVSW			;initiate the transfer
;
; restore the contents of location 800h
;
	MOV	ES,AX
	XOR	DI,DI
	PUSH	CS
	POP	DS
	MOV	SI,OFFSET save_adr800_blk
	MOV	CX,0033H
	REPZ	MOVSW			;cs:2b9 -> ??:0 33h times
	STI

L0221:	MOV	CX,0800h
L0224:	MOV	AX,0DD00h		;
	CALL	command_8042		;ah=command
	LOOPNZ	L0224
do_transfer_exit:
	pop	bp
	ret
@ext_transfer_error:
	jmp	ext_transfer_error
;-----------------------------------
;  ah=command for port 60h  al=att command ?
command_8042:
	CLI
	CALL	wait_8042_inbuf_ful
	JNZ	L024C
	MOV	AL,0D1H
	OUT	64H,AL			;force next out to 60 to be cmd
	CALL	wait_8042_inbuf_ful
	JNZ	L024C
	MOV	AL,AH
	OUT	60H,AL			;see notes
	CALL	wait_8042_inbuf_ful
L024C:	STI
	RET

wait_8042_inbuf_ful:
	XOR	CX,CX
L02A1:	IN	AL,64H			;read 8042 status
	AND	AL,2			;check 8042	input buffer full
	LOOPNZ	L02A1
	RET
;-------------------------
; ax*16	then store at ds:si  &	ds:si+2
;
seg_to_abs:
	push	cx
	MOV	CX,010H
	MUL	CX
	MOV	[SI],AX
	MOV	[SI+2],DL
	pop	cx
	RET
;-------------------------------------------------------------------------
;-----------------------------------------------------------------------------
FREE_1K_BLOCKS	DW	0		;number of	free 256k blocks
;
; initialize the EXT interface
;
setup_ext_memory:
	call	get_processor_type
;
; save int15 address just incase of reboot
;
	push	es
	mov	ax,3515h
	int	21h
	mov	int15_off,bx
	mov	int15_seg,es
	pop	es

		call	check_extended
	jc	extended_exists		;jump if extended memory found
ext_error:
	mov	dx,offset err6		;get error	message
	jmp	err_exit
;
; check	if we have enough expanded memory
;
extended_exists:
	cmp	extended_flag,3		;check if	xms allocation
	je	verify_avail		; jmp	if xms (skip force check)
	cmp	memory_start_force,1
	jne	verify_avail
;
; check	if start within	range
;
	mov	ax,starting_extendedl
	mov	dl,starting_extendedh
	mov	dh,0
	div	c1024			;compute forced starting 1k adr
	add	ax,users_size_request	;compute final allocation adr.

	mov	bx,M_EXTENDED_SIZE
	add	bx,1024		      ;compute	final ext. mem.	adr
	cmp	ax,bx
	jb	setup_int19
	mov	dx,offset bad_strt_msg
	jmp	err_exit

verify_avail:
	mov	ax,FREE_1K_BLOCKS	;get amount of memory available
	sub	ax,users_size_request	;check if we	can satisify user request
	jns	setup_int19	      ;jump if	we have	enough extended	mem
	mov	dx,offset err5
	jmp	err_exit
setup_int19:
	sub	ax,ax
	mov	es,ax
;
; save origional offset	& store	our offset into	vector
;
	cli
	mov	si,(4*19h)			    ;setup pointer to	int19
	mov	ax,es:[si]		;get orig. offset
	mov	ORIG_INT19_OFFSET2,ax	;save it
	mov	word ptr es:[si],(offset our_int19) +100h
;
; save origional segment & store our segment into vector
;
	add	si,2			;move to segment
	mov	ax,es:[si]		;get orig. segment
	mov	ORIG_INT19_SEGMENT2,ax	;save it
	mov	ax,cs			;compute segment
	sub	ax,10h
	mov	es:[si],ax		;segment for our int19	routine
	sti
	cmp	memory_start_force,1
	je	update_ext_mem
;
; check	if xms allocate
;
	cmp	extended_flag,3
	jne	check_for_high
	mov	dx,users_size_request
;
; allocate extended memory block
;  inputs: dx -	size needed in k-bytes
;  output: ax =	0001 if	block allocated	ok
;	   dx =	handle for block allocated
; Note:	handle is stored for use and can be ignored.
;
allocate_xms:
	mov	ah,9
	call	xms_control
	mov	xms_from_read_handle,dx
	mov	xms_to_write_handle,dx

	jmp	bypass_auto_ext
;
; check	if high	memory use request
;
check_for_high:
	cmp	high_memory_flag,1
	jne	update_ext_mem		;jmp if low memory	ussage
	call	grab_high_memory
	jmp	ext_exita
;
; update free memory in	device driver pointed at by int19 (it may be us)
;
update_ext_mem:
	sub	ax,ax
	mov	es,ax
	mov	si,(4*19h)		;get ptr. to vector for int19
	les	si,es:[si]		;get vector contents in es:si
	mov	si,AVAIL_LO		;get offset for start	of free	memory
	mov	ax,users_size_request	;get	# of 1k	blocks user requested
	mul	C1024			;convert to bytes in dx,ax
	add	es:[si],ax		;add our size to
	adc	es:[si+2],dl		;   free memory in device driver
ext_exita:
;
; set memory_type to bios transfer mode	(3) if bios faster than	loadall
; The default state of memory_type is loadall (2) unless  a). xms driver
; is found  b).	the param* parameter was found	c). the	processor type
; is 386.
;
	cmp	memory_type,7		;check if bios/xms force
	je	bypass_auto_ext		; jmp if bios/xms force
	cmp	processor_type,1	;check if	286 processor
	jne	bypass_auto_ext		;	jmp if 80386 or	other
	cmp	ps2_flag,0		;check	if bios	force request
	je	check_spd
	mov	memory_type,7		;force xms/bios if ps/2
	jmp	bypass_auto_ext
check_spd:
	call	check_extended_speed
bypass_auto_ext:
;
; grab the keyboard interrupt
;   The	keyboard is check for ctrl-alt-del and if found	the subroutine
;   reboot is called.  This may	be needed for the case where we	are
;   in the middle of a memory move & the processor state will not support
;   a warm reboot.
;
	MOV	AX,3509h
	INT	21H			;get	int09 vector (keyboard)
	MOV	key_vector,BX
	MOV	key_seg,ES
	MOV	DX,OFFSET int09
	MOV	AX,2509h
	INT	21H			;set	int09 vector (keyboard)
	mov	dx,0			;indicate no errors
err_exit:
	mov	ax,cs
	mov	es,ax
	ret
;---------------------------------------------
grab_high_memory:
	mov	ax,FREE_1K_BLOCKS	;get extended size in 1k	blocks
	mul	c1024			;comvert to byte count (dx,ax)
	mov	cx,dx			;save size
	mov	bx,ax			;	in (cx,bx)
	mov	ax,users_size_request
	mul	c1024			;compute request in bytes
;
	sub	bx,ax			;compute memory
	sbb	cx,dx			;	remaining (cx,bx)

	add	starting_extendedl,bx	;compute our
	adc	starting_extendedh,cl	;  starting point

	mov	ax,users_size_request
	sub	EXTENDED_SIZE,ax	;adjust size for next user
;
	mov	ax,3515h
	int	21h			;get	current	int15 vector
	mov	int15_off,bx
	mov	int15_seg,es
	mov	dx,offset int15_trap
	mov	ax,2515h
	int	21h
	mov	ax,cs
	mov	es,ax
	ret
;---------------------------------------------

; Subroutine to	check if extended memory exists
;   inputs - none
;   output - carry set if found
;	     STARTING_EXTENDED = set to	starting address of extended memory
;	     EXTENDED_SIZE     = set to	size of	extended memory	in 1k blocks
;	     EXTENDED_FLAG     = 0=no ext 1=ext	found  2=ext & device drive found
;
check_extended:
;
; check	if another device driver has been installed for	extended memory
;
	call	check_for_xms		;check for	xms driver
	cmp	bx,0			;
	je	check_ext_sz		;jmp if no xms driver
	mov	word ptr [xms_control],bx
	mov	word ptr [xms_control+2],es
	call	get_xms_status		;check available memory, dx=total	avail.
	mov	free_1k_blocks,ax
	mov	extended_flag,3		;set xms flag
;
; is this is an	old xms	driver,it reports the HMA memory area as part of
; above	available block.  This area is not actually available.	So,
; If XMS version 2.5 or	earlier	then assume we may need	to grab	the
; HMA area also.
;
	mov	ah,0
	call	xms_control		;get	version	in -bx-
	cmp	bx,0205h
	ja	xms_cont		;jmp if version ok
;
; warn user if older version of	xms
;
	mov	dx,offset xms_warn
	mov	ah,9
	int	21h

	mov	ax,users_size_request
	add	ax,40h			;size requested +	64k
	cmp	ax,free_1k_blocks
	jbe	xms_cont		;jmp if no confilict
;
; we are using part of HMA area, so reserve it also.
;
	mov	ah,1
	mov	dx,0fff0h			;reserve total	64k-16 of HMA
	call	xms_control
xms_cont:
	jmp	bypass_loadall_setup
;
; check	total size of extended memory by calling bios
;
check_ext_sz:
	clc				;clear carry
	mov	ah,88h			;get extended memory size	code
	int	15h			;call bios
	jnc	cont_ext1		;jump if we may	have extended memory
	jmp	no_extended		;jump	if no extended installed
cont_ext1:
	mov	EXTENDED_SIZE,ax	;save size of extended menory

check_for_vdisk:
	sub	ax,ax
	mov	es,ax			;-es- now points to low ram
	mov	si,(4*19h)+2	    ;get offset of int19 handlers segment
	cmp	word ptr es:[si],70h	;check if dos
	jne	check_driver_contents
	jmp	no_extended_drivers
check_driver_contents:
	mov	si,(4*19h)
	les	si,es:[si]		;point	es:si at device	driver
	mov	si,RESERVED		;check reserved area also
	cmp	word ptr es:[si],0
	je	cont_ext3		;jump if	expected value found
	jmp	no_extended_drivers
cont_ext3:
	mov	si,AVAIL_LO		;get pointer to end of mem. used so far
	cmp	byte ptr es:[si+2],10h
	jb	no_extended_drivers
	mov	ax,es:[si]		;get low word of address
	mov	STARTING_EXTENDEDl,ax	;save start of avail	memory
	add	si,2
	mov	al,es:[si]		;get high byte	of data
	mov	STARTING_EXTENDEDh,al	;save start of avail	memory
;
; compute available memory (total size - previous users	size)
;
	mov	ax,EXTENDED_SIZE	;get size	of extended memory
	mul	C1024			;convert to byte count
	add	dx,10h			;compute end of memory byte count
	sub	ax,STARTING_EXTENDEDl	;compute amount
	sbb	dl,STARTING_EXTENDEDh	;  of free memory from
	cmp	dx,c1024
	jnb	divide_err3		;jump	if divide overflow
	div	C1024				 ;    total - last users amount
	mov	FREE_1K_BLOCKS,ax		  ;save total # of free 1k blocks
	mov	EXTENDED_FLAG,2		;set extended & driver found
	jmp	check_extended_exit
no_extended_drivers:
	mov	EXTENDED_FLAG,1		;set extended exists, no drivers
	call	check_for_fASTdisk	;check	if AST driver is installed
	jnc	no_ast_drivers
	mov	ax,EXTENDED_SIZE	;get size	of extended memory
	mul	C1024			;convert to byte count
	add	dx,10h			;compute end of memory byte count
	sub	ax,STARTING_EXTENDEDl	;compute amount
	sbb	dl,STARTING_EXTENDEDh	;  of free memory from
	cmp	dx,1024
	jnb	divide_err3
	div	C1024				 ;    total - last users amount
	mov	FREE_1K_BLOCKS,ax		  ;save total # of free 1k blocks
	jmp	check_extended_exit
divide_err3:
	mov	al,3
	jmp	late_abort
no_ast_drivers:
	mov	ax,EXTENDED_SIZE	;get size	of extended memory
;
; total	size of	remaing	extended memory	in 1k blocks is	in -ax-
; we can assume	that we	are the	only user so far
;
	mov	FREE_1K_BLOCKS,ax	;save amount available for our use
check_extended_exit:
;
; extended memory setup
;
ext_mem_init:
	cmp	processor_type,1		;check if 286
	jne	bypass_loadall_setup
	cmp	ps2_flag,0
	jne	bypass_loadall_setup		;skip loadall setup if ps/2
	cmp	memory_type,7
	je	bypass_loadall_setup		;skip	loadall	if bios/xms force
	smsw	machine_status_word
;L0F19:	DB	0FH,1			;CTS	  01
;	DB	26H
;	DW	machine_status_word

;	sidt	interrupt_descriptor_table
	DB	0FH,1			;CTS    01
	DB	0EH
	DW	interrupt_descriptor_table

;	sgdt	global_descriptor_table
	DB	0FH,1			;CTS    01
	DB	6
	DW	global_descriptor_table

	MOV	SI,OFFSET interrupt_descriptor_table
	CALL	shuffle_address_bytes
	MOV	SI,OFFSET global_descriptor_table
	CALL	shuffle_address_bytes
	MOV	cs_selector,CS
	MOV	SI,OFFSET cs_descriptor
	MOV	AX,CS
	CALL	seg_to_abs		;ax*16 -> ds:si
bypass_loadall_setup:
	stc				;set extended found
	jmp	ext_exit
no_extended:
	mov	EXTENDED_FLAG,0
	clc
ext_exit:
	mov	bx,cs
	mov	es,bx
	ret

shuffle_address_bytes:
	PUSH	[SI+4]
	PUSH	[SI+2]
	PUSH	[SI]
	POP	[SI+4]
	POP	[SI]
	POP	[SI+2]
	RET
;------------------------------------------
; this proceedure checks for AST's FASTDISK
;
AST_DATA  label	 byte
 START_EXTENDED_MEMORY	DW	0		;paragraph address
			 DW	0
 MEM_TYPE_FLAG		     DW	0		;set to 0ffff for extended
			 DW	0
 FRE_EXTENDED_MEMORYL	DW	0		;paragraph address
 FRE_EXTENDED_MEMORYH	DW	0
;
;
check_for_fASTdisk  proc  near
	cld
	sub	ax,ax
	mov	es,ax		;set initial segment to 0
chk_loop1:
	mov	di,0		;set	initial	offset
	mov	al,0deh		;get first code
	mov	cx,4000h	;get search size
chk_loop2:
	repne	scasb		;scan for	code1
	or	cx,cx
	jz	nxt_partition	;jump	if no match found
;
; first	char. matched, check next char.
;
	cmp	byte ptr es:[di],'f'	;check next code
	jnz	chk_loop2		;jump if no match
;
; second char. matched,	check next char.
;
	inc	di
	dec	cx
	jz	nxt_partition		;jump if at end of partition
	cmp	byte ptr es:[di],'A'	;check next code
	jz	chk_third		;jump if	no match
	dec	di			;move	back 1
	inc	cx			;move	back 1
;
; third	char. matched, check last char.
;
chk_third:
	inc	di
	dec	cx
	jz	nxt_partition
	cmp	byte ptr es:[di],'S'
	jnz	chk_loop2	;jump if	no match
;
; we found a match - at	- es:[di-4]  now move to parameters
;
	and	di,0fff0h	;move to	start of line
	mov	ax,es		;get segment
	sub	ax,6		;move back to start of data
	mov	es,ax		;
	mov	si,offset AST_DATA
	mov	cx,12
move_ast_lp:
	mov	al,es:[di]
	mov	[si],al
	inc	di
	inc	si
	loop	move_ast_lp
;
; verify it is an extended memory driver
;
	cmp	FRE_EXTENDED_MEMORYH,-1
	je	not_foundd			;jump if not extended driver
;
; get start of available and convert to	byte addresss
;
	mov	ax,FRE_EXTENDED_MEMORYl
	mul	C16
	mov	STARTING_EXTENDEDl,ax

	mov	ax,FRE_EXTENDED_MEMORYh
	shl	ax,1
	shl	ax,1
	shl	ax,1
	shl	ax,1
	add	al,dl
	mov	STARTING_EXTENDEDh,al

;
; compute amount of extended memory avail.
;
have_match:
	stc
	ret
;
; move to nxt partition	(leave a 16byte	overlap)
;
nxt_partition:
	mov	ax,es
	mov	bx,cs		;get our segment
	cmp	bx,ax		;check if all memory scanned
	jle	not_foundd	;jump if all memory scanned
	add	ax,3ffh		;move segment forward
	mov	es,ax
	jmp	chk_loop1	;go back	and look at rest of memory
;
; no match was foundd -	initialize our match area
;
not_foundd:
	clc			;clear the carry
	ret
check_for_fASTdisk	endp

;----------------------------------------------------------------
; check	if XMS eXtended	Memory Specification driver exists
;  inputs:  none
;  output:  es:bx = XMS	control	address	if found
;	       bx = zero if no XMS driver found
;
; Note:	to save	control	point -	mov word ptr [xms_control],bx
;				mov word ptr [xms_control+2],es
;
;	control	save definded as - xms_control	dd   0
;
check_for_xms:
	mov	ax,4300h
	int	2fh
	cmp	al,80h		;check if driver found
	mov	bx,0		;pre	load failure code
	jne	xms_check_exit	;jmp if driver not found
;
	mov	ax,4310h
	int	2fh		;request control address
xms_check_exit:
	ret
;---------------------------------------------------------------
; request amount of free extended memory
;  inputs: none
;  output: ax =	size of	largest	block available	in k-bytes
;	   dx =	total size of avail. memory in k-bytes
;
get_xms_status:
	mov	ah,8
	call	xms_control
	ret
;--------------------------------------------------------------------------
; check	extended memory	speed and set memory_type -> 3 if bios is faster
; than the loadall logic.
;
starting_cursor	dw	0
loadall_count	dw	0
bios_count	dw	0

check_extended_speed:
	mov	ax,cs
	mov	es,ax

	mov	ah,3
	mov	bh,0
	int	10h		;read	cursor position	to -dx-
	mov	starting_cursor,dx
;
; display information, incase the loadall fails
;
	mov	si,offset bios_force_msg
drr_lp:	lodsb
	cmp	al,'$'
	je	m_done			      ;jmp if msg displayed
	mov	ah,0eh
	mov	bh,0
	push	si
	int	10h
	pop	si
	jmp	drr_lp
m_done:
;
; do preliminary loadall to verify it works
;  memory_type flag is assumed to be set to loadall mode (2)
;
	mov	di,(offset setup_stack)+2	;use end	of program as buffer
	mov	ax,1				;read adr.	1meg
	mov	cx,256				;read 256 words
	call	copy_from_program
;
; if we	got here then the extended read	was sucessful. zap the message displayed.
;
	mov	ah,2
	mov	bh,0
	mov	dx,starting_cursor
	int	10h				;restore cursor posn

	mov	ax,0a20h
	mov	bl,0
	mov	cx,70
	int	10h				;zap last message
;
; wait for tick	counter	to be bumped.
;
	call	wait_for_timer_bump		;timer bit 010b is =	0
;
; read extended	memory using loadall until next	timer bump
;
	mov	memory_type,7			;force bios transfers
bios_loop:
	mov	ax,cs
	mov	es,ax
	mov	di,(offset setup_stack)+2	;use end	of program as buffer
	mov	ax,1				;read adr.	1meg
	mov	cx,256				;read 256 words
	call	copy_from_program
	inc	bios_count
	sub	ax,ax
	mov	es,ax
	test	byte ptr es:[46ch],02h
	jz	bios_loop
;
; wait for tick	counter	to be bumped.
;
	call	wait_for_timer_bump		;timer bit 010b is =	0
;
; read extended	memory using loadall until next	timer bump
;
	mov	memory_type,6			;force loadall transfers
loadall_loop:
	mov	ax,cs
	mov	es,ax
	mov	di,(offset setup_stack)+2	;use end	of program as buffer
	mov	ax,1				;read adr.	1meg
	mov	cx,256				;read 256 words
	call	copy_from_program
	inc	loadall_count
	sub	ax,ax
	mov	es,ax
	test	byte ptr es:[46ch],02h
	jz	loadall_loop
;
; now compare counts to	see which is faster
;
	mov	ax,loadall_count
	cmp	ax,bios_count
	ja	ces_exit			;jmp if loadall faster
	mov	memory_type,7			;switch to	bios mode
ces_exit:
	ret
;---------------------------------
wait_for_timer_bump:
	sub	ax,ax
	mov	es,ax
wft1:	test	byte	ptr es:[46ch],02h
	jz	wft1
wft2:	test	byte	ptr es:[46ch],02h
	jnz	wft2
	ret


