;********************** XXMS.asm ************************
;
; This program demonstrates the calls for interrogating XMS
; memory presence and current state.  It also shows the
; state of the A20 line.  Jeff Owens 10-4-94
;
code	segment
	assume	cs:code,ds:code

xms_control		dd	0

total_xms_1k_blks	dw	0
contigious_xms_1k_blks	dw	0

xms_version		dw	0

bios_extended_size	dw	0

xms_handle		dw	0

no_xms_msg	db	0dh,0ah
		db	'XMS memory not found$'
xms_found_msg	db	0dh,0ah
		db	'XMS memory found$'
contig_msg	db	0dh,0ah
		db	'Contigious xms 1k blocks = $'
total_msg	db	0dh,0ah
		db	'Total xms 1k blocks = $'
xms_version_msg	db	0dh,0ah
		db	'XMS version = $'
bios_ext_msg	db	0dh,0ah
		db	'BIOS reports extended memory = $'
umb_msg		db	0dh,0ah
		db	'Upper memory area is available.$'
wrap_msg	db	0dh,0ah
		db	'A20 line is disabled (memory wraps at 1meg)$'
no_wrap_msg	db	0dh,0ah
		db	'A20 line is enabled (memory does not wrap at 1meg)$'
xms_a20_on	db	0dh,0ah
		db	'XMS reports A20 enabled (no wrap)$'
xms_a20_off	db	0dh,0ah
		db	'XMS reports A20 disabled (wrapping)$'
xms_a20_err1	db	0dh,0ah
		db	'XMS reports VDISK has control of A20$'
xms_a20_err2	db	0dh,0ah
		db	'XMS A20 status reports error$'
crlf		db	0dh,0ah,'$'
;---------------------------------------------------------------------------

start:
	mov	ax,cs
	mov	ds,ax
	mov	es,ax
	cli
	mov	ss,ax
	mov	sp,offset stack_
	sti

	call	check_extended
	jc	extended_exists		;jump if extended memory found
ext_error:
	mov	dx,offset no_xms_msg		;get error message
	mov	ah,9
	int	21h
;
; check a20 line status
;
extended_exists:
	push	ds
	push	es
	mov	ax,0
	mov	es,ax
	dec	ax
	mov	ds,ax
	mov	ax,word ptr es:[4eh]
	cmp	ax,word ptr ds:[5eh]
	jne	no_wrap
	mov	word ptr ds:[5eh],1234h
	cmp	word ptr es:[4eh],1234h
	mov	word ptr ds:[5eh],ax
	jne	no_wrap
is_wraping:
	pop	es
	pop	ds
	mov	dx,offset wrap_msg
	jmp	wrap_display
no_wrap:
	pop	es
	pop	ds
	mov	dx,offset no_wrap_msg
wrap_display:
	mov	ah,9
	int	21h
	jmp	xxms_exit
;
; allocate extended memory block
;  inputs: dx -	size needed in k-bytes
;  output: ax =	0001 if	block allocated	ok
;	   dx =	handle for block allocated
; Note:	handle is stored for use and can be ignored.
;
allocate_xms:
	mov	ah,9
	call	xms_control
	mov	xms_handle,dx

xxms_exit:
	mov	ah,9
	mov	dx,offset crlf
	int	21h
	mov	ax,4c00h
	int	21h
	
;---------------------------------------------

; Subroutine to	check if extended memory exists
;   inputs - none
;   output - carry set if found
;	     EXTENDED_SIZE     = set to	size of	extended memory	in 1k blocks
;
check_extended:
;
; check	if another device driver has been installed for	extended memory
;
	call	check_for_xms		;check for xms driver
	cmp	bx,0			;
	jne	xms_is_here
	jmp	check_ext_sz		;jmp if no xms driver
xms_is_here:
	mov	word ptr [xms_control],bx
	mov	word ptr [xms_control+2],es

	mov	ah,9
	mov	dx,offset xms_found_msg
	int	21h
	
	call	get_xms_status		;check available memory, dx=total avail.
	mov	total_xms_1k_blks,dx
	mov	contigious_xms_1k_blks,ax

	mov	ah,9
	mov	dx,offset contig_msg
	int	21h
	mov	ax,contigious_xms_1k_blks
	call	display_binary

	mov	ah,9
	mov	dx,offset total_msg
	int	21h
	mov	ax,total_xms_1k_blks
	call	display_binary
	
;
; is this is an	old xms	driver,it reports the HMA memory area as part of
; above	available block.  This area is not actually available.	So,
; If XMS version 2.5 or	earlier	then assume we may need	to grab	the
; HMA area also.
;
	mov	ah,0
	call	xms_control		;get	version	in -bx-
	mov	xms_version,bx

	mov	ah,9
	mov	dx,offset xms_version_msg
	int	21h
	mov	ax,bx
	call	hex_word_to_ascii
check_for_umb:
	mov	ah,10h
	mov	dx,4000h
	call	xms_control
	cmp	bl,0b0h
	jne	no_umb
	mov	ah,9
	mov	dx,offset umb_msg
	int	21h
no_umb:
;
; querry the xms driver for state of A20 line
;
	mov	ah,07
	call	xms_control
	cmp	bl,00
	jne	xms_a20_err
	mov	dx,offset xms_a20_on
	cmp	ax,1
	je	xms_a20_display
	mov	dx,offset xms_a20_off
	jmp	xms_a20_display
xms_a20_err:
	mov	dx,offset xms_a20_err1
	cmp	bl,81h
	je	xms_a20_display
	mov	dx,offset xms_a20_err2
xms_a20_display:
	mov	ah,9
	int	21h
	

		
	clc				;clear carry
	mov	ah,88h			;get extended memory size code
	int	15h			;call bios
	jc	ce_exit			;jmp if no bios extended
	mov	bios_extended_size,ax

	mov	ah,9
	mov	dx,offset bios_ext_msg
	int	21h
	mov	ax,bios_extended_size
	call	display_binary
	stc
	jmp	ce_exit
;
; check	total size of extended memory by calling bios
;
check_ext_sz:
	clc				;clear carry
	mov	ah,88h			;get extended memory size	code
	int	15h			;call bios
	jnc	cont_ext1		;jump if we may	have extended memory
	clc
	jmp	ce_exit    		;jump	if no extended installed
cont_ext1:
	mov	bios_EXTENDED_SIZE,ax	;save size of extended menory
ce_exit:
	ret
;----------------------------------------------------------------
; check	if XMS eXtended	Memory Specification driver exists
;  inputs:  none
;  output:  es:bx = XMS	control	address	if found
;	       bx = zero if no XMS driver found
;
; Note:	to save	control	point -	mov word ptr [xms_control],bx
;				mov word ptr [xms_control+2],es
;
;	control	save definded as - xms_control	dd   0
;
check_for_xms:
	mov	ax,4300h
	int	2fh
	cmp	al,80h		;check if driver found
	mov	bx,0		;pre	load failure code
	jne	xms_check_exit	;jmp if driver not found
;
	mov	ax,4310h
	int	2fh		;request control address
xms_check_exit:
	ret
;---------------------------------------------------------------
; request amount of free extended memory
;  inputs: none
;  output: ax =	size of	largest	block available	in k-bytes
;	   dx =	total size of avail. memory in k-bytes
;
get_xms_status:
	mov	ah,8
	call	xms_control
	ret


;***********************  CONVERT *******************************

ten		dw	10

;============================================================================
display_char:	push	bx
		push	ax
		mov	ah,0eh			;tty display
		mov	bh,0			;page
		int	10h
		pop	ax
		pop	bx
		ret				
;=============================================================================
; input dx,ax = hex dword

hex_dword_to_ascii:
		xchg	ax,dx
		call	hex_word_to_ascii
		xchg	ax,dx
;=============================================================================
; ax = hex word
;
hex_word_to_ascii		proc	near
		xchg	ah,al
		call	hex_byte_to_ascii	
		xchg	ah,al
hex_byte_to_ascii:
		mov	cl,4
		ror	al,cl
		call	hex_nibble_to_ascii
		ror	al,cl
hex_word_to_ascii		endp
  
;==========================================================================
  
hex_nibble_to_ascii		proc	near
		push	ax
		and	al,0Fh
		cmp	al,0Ah
		jb	loc_69
		add	al,7
loc_69:		add	al,'0'
		call	display_char
		pop	ax
		retn
hex_nibble_to_ascii		endp
;-------------------------------------------------------------------------
; inputs al = binary
;
byte_to_decimal:
	push	ax
	aam			;convert byte to bcd
	or	ax,3030h	;add in ascii
	xchg	al,ah
	call	display_char
	xchg	al,ah
	call	display_char
	pop	ax
	ret  
  
;==========================================================================
;  inputs:  ax = binary value
;  outputs: none
;
;  registers destroyed:  ax,dx
;
  
display_binary:	xor	dx,dx
		div	cs:ten			;divide curent binary val by 10
		or	ax,ax			;check if done
		jz	db_display		;  jmp if done
		push	dx			;save remainder
		call	display_binary		;recursion
		pop	dx
db_display:	add	dl,30h			;convert char. to ascii
		mov	ah,2
		int	21h			;display char. in -dl-
		ret

;---------------------------------------------------------------------------
  
;==========================================================================
;  inputs:  dx,ax = binary value
;  outputs: none
;
;  registers destroyed:  ax,dx
;
  
ddisplay_binary:
		push	ax
		mov	ax,dx
		xor	dx,dx
		div	cs:ten
		mov	bx,ax
		pop	ax
		div	cs:ten
		xchg	bx,dx
;
; dx,ax = quotient
;    bx = remainder

		or	ax,ax
		jnz	dd_more
		or	dx,dx
		jnz	dd_more
		mov	dx,bx
		jmp	ddb_display
dd_more:
		push	bx			;save remainder		
	
		call	ddisplay_binary		;recursion
		pop	dx
ddb_display:	add	dl,30h			;convert char. to ascii
		mov	ah,2
		int	21h			;display char. in -dl-
		ret

;---------------------------------------------------------------------------

;-------------------------------------------------------------------------
	dw	100	dup (?)
stack_	dw	?

code	ends
	end	start
