_
DEC16IN

  Function: This routine accepts a decimal number from the standard input
    device and converts it to internal 16-bit binary form.

  Input: The individual digits of the decimal number are received in ASCII
    through a call to a standard I/O routine.  The valid digits are
    0 through 0. An ASCII code other than for a valid digit will terminate
   the routine.

  Output: A 16-bit binary number is returned in the DX register.

  Registers used: Only DX is modified. It returns the result.

  Routines called: STDIN
______________________________________________________________________________
;-----------------------------------------------------------------------------
;Routine to convert from ASCII decimal to internal 16-bit binary
;
DEC16IN   PROC	 FAR
	  PUSH	 CX		       ;Save registers
	  PUSH	 AX
	  MOV	 DX,0		       ;Initialize DX as 0
DEC16IN1:
	  CALL	 STDIN		       ;Digit comes in AL
	  SUB	 AL,30H 	       ;Subtract 30H
	  JL	 DEC16IN2	       ;Check if too low
	  CMP	 AL,9
	  JG	 DEC16IN2	       ;Check if too high
	  CBW			       ;Convert to word
	  PUSH	 AX		       ;Save digit
	  MOV	 AX,DX
	  MOV	 CX,10		       ;Multiplier of 10
	  MUL	 CX		       ;Multiply
	  MOV	 DX,AX		       ;Result in DX
	  POP	 AX		       ;Restore digit
	  ADD	 DX,AX		       ;Add in digit
	  JMP	 DEC16IN1
DEC16IN2:
	  POP	 AX		       ;Restore registers
	  POP	 CX
DEC16IN   ENDP

ten		dw	10
  
;==========================================================================
;  inputs:  ds:si - points at ascii string ending with 0-2fh , 3ah-0ffh
;  output:  ax    - binary value
;
;  registers destryed:  ax,bx,dx
;  
ascii_to_binary:xor	bx,bx			;clear sum
atb_loop:	lodsb				;get next char
		sub	al,30h			; '0'
		jc	atb_exit		;jmp if 0-2fh (end of string)
		cmp	al,9
		ja	atb_exit		;jmp if 3ah-0ffh (end of string)
		cbw
		xchg	ax,bx
		mul	cs:ten			;mul sum * 10
		add	bx,ax			;add current char. value
		jmp	short atb_loop
		
atb_exit:	mov	ax,bx
		ret
  

	page	66,132
;******************************** CONV01.ASM *********************************

LIBSEG           segment byte public "LIB"
		assume cs:LIBSEG , ds:nothing

;----------------------------------------------------------------------------
.xlist
	include  mac.inc
	include  common.inc
.list
comment 
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -( CONVERT )
DEC_STR_TO_WORD -  convert unsigned ascii string to binary word
;
; inputs  ds:si points	at string ending with "space" or "return"
; 
; output  ax = binary value
;	   si = updated to point at end of value
;	   carry set if	error
;* * * * * * * * * * * * * *

ab_low	dw	0

	PUBLIC	DEC_STR_TO_WORD
DEC_STR_TO_WORD		PROC	FAR
	push	di
	push	bx
	mov	cs:ab_low,0			;  accumulators
;
; remove blanks in front of number
;
	mov	bx,10			;base 10?
ab1:	cmp	byte ptr [si],' '
	jne	ab2			;jmp if not preceeding space
nxt_flush:
	inc	si
	jmp	ab1
;
; conversion loop
;	
ab_loop:
	inc	si			;move	to next	char
ab2:	mov	al,[si]
;
; convert key to binary
;
key21:	cmp	al,0dh
	je	ab_done
	cmp	al,' '
	je	ab_done
	cmp	al,09
	je	ab_done
	cmp	al,'/'
	je	ab_done
	cmp	al,0
	je	ab_done
key22:	cmp	al,'0'
	jl	ab_err			;skip non numeric key
	cmp	al,'9'
	jle	decimal			;jump if	decimal	value
;
; we have a hex	number or bad value.
;
	jmp	ab_err			;	jmp if wrong base
decimal:
	sub	al,'0'			;convert to number
got_binary_key:
	sub	ah,ah
	push	ax			;save key
;
; multiply sum by current base
;
	mov	ax,cs:ab_low
	mov	dx,0
	mul	bx
;
; add latest character to sum
;
	pop	di
	add	ax,di			;add to sum
	mov	di,0
	adc	dx,di			;add carry
;
	mov	cs:ab_low,ax
	jmp	ab_loop
ab_err:
	stc
	jmp	ab_exit
ab_done:
	mov	ax,cs:ab_low
	clc
ab_exit:
	pop	bx
	pop	di
	RETF
DEC_STR_TO_WORD ENDP

LIBSEG	ENDS
	end
