
>;===============================================================
>; Subroutine to convert a binary value to decimal and display it.
>;  inputs:  ax = binary value
>;  outputs: none
>;
>;  registers destroyed:  ax,dx
>;
>  
You sacrifice a lot of speed for one byte of memory and one register if you use
                   mov   bx,10
>display_binary:   xor   dx,dx
>                  div   bx  ; cs:ten         ;divide value by 10 <in bx
>                  test  ax,ax          ;check if done
>                  push  dx             ;save remainder
>                  jz    db_display     ;jmp if done
>                  call  display_binary ;recursion
>db_display:       pop   ax
>                  add   al,30h
>                  int   29h
>                  ret
>  
>ten               dw 10

DEC16OUT

  Function: This routine accepts a 16-bit binary number in the DX register,
    converts it to ASCII decimal form and sends it to the standard output
    device.

  Input: Upon entry a 16-bit binary number is in the DX register

  Output: A string of ASCII digits representing a decimal number is stored in
    a buffer called TBUFF and then sent out through the standard output device.

  Registers used: No registers are modified. DX is used for input.

  Segments referenced: DATAS is data segment which contains TBUFF

  Routines called: STDOUT
;-----------------------------------------------------------------------------
;Routine to convert from internal 16-bit binary to ASCII decimal
;
DEC16OUT  PROC	 FAR
;
;A binary number is in DX
	  PUSH	 DS		       ;Save registers
	  PUSH	 DI
	  PUSH	 DX
	  PUSH	 CX
	  PUSH	 AX
	  MOV	 AX,DATAS	       ;Point to data segment
	  MOV	 DS,AX
;
;Put the digits in a buffer
	  MOV	 CX,0		       ;Initialize the counter
	  LEA	 DI,TBUFF	       ;Point to the buffer
DEC16OUT1:
	  PUSH	 CX		       ;Save CX
	  MOV	 AX,DX		       ;Numerator
	  MOV	 DX,0		       ;Clear upper half
	  MOV	 CX,10		       ;Divisor of 10
	  DIV	 CX		       ;Divide
	  XCHG	 AX,DX		       ;Get quotient
	  ADD	 AL,30H 	       ;Add 30H
	  MOV	 [DI],AL	       ;Put in buffer
	  INC	 DI		       ;Next byte
	  POP	 CX		       ;Restore CX
	  INC	 CX		       ;Count the digit
	  CMP   DX,0		       ;Done?
	  JNZ	 DEC16OUT1
;
;Dump the buffer out
DEC16OUT2:
	  DEC	 DI		       ;Back up through the buffer
	  MOV	 AL,[DI]	       ;Get byte from buffer
	  CALL	 STDOUT 	       ;Send it out
	  LOOP	 DEC16OUT2
	  POP	 AX		       ;Restore registers
	  POP	 CX
	  POP	 DX
	  POP	 DI
	  POP	 DS
	  RET			       ;Return
DEC16OUT  ENDP
;-----------------------------------------------------------------------------
  
;==========================================================================
;  inputs:  ax = binary value
;  outputs: none
;
;  registers destroyed:  ax,dx
;
  
display_binary:	xor	dx,dx
		div	cs:ten			;divide curent binary val by 10
		or	ax,ax			;check if done
		jz	db_display		;  jmp if done
		push	dx			;save remainder
		call	display_binary		;recursion
		pop	dx
db_display:	add	dl,30h			;convert char. to ascii
		mov	ah,2
		int	21h			;display char. in -dl-
		ret

;---------------------------------------------------------------------------


	page	66,132
;******************************** CONV04.ASM *********************************

LIBSEG           segment byte public "LIB"
		assume cs:LIBSEG , ds:nothing

;----------------------------------------------------------------------------
.xlist
	include  mac.inc
	include  common.inc
.list
;----------------------------------------------------------------------------
	extrn	lib_info:near
comment 
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -( CONVERT )
WORD_TO_DEC_STR - convert an integer value to an ASCIIZ string
;
; inputs:    DS:[SI] pointing to a buffer space
;            (WORD_TO_DEC_STR) AX = integer value
;            WORD_TO_DEC_STR requires a 7-byte (or greater) buffer;
;            
; output:    ASCIIZ string at DS:[SI]; numerals are right-justified
;* * * * * * * * * * * * * *

	PUBLIC	WORD_TO_DEC_STR
WORD_TO_DEC_STR	PROC	FAR
    APUSH   AX,BX,CX,DX,DI,SI,ES,DS
    POP     ES
    MOV     DI,SI
    CLD
    MOV     SI,AX
    MOV     CX,0006
    MOV     AL,20h    ; ' '
    REPZ    STOSB
    MOV     ES:[DI],CL
    MOV     AX,SI
    MOV     BX,000Ah
    OR      SI,SI
    JNS     wts_cont
    NEG     AX
wts_cont:    
    XOR     DX,DX
    DIV     BX
    ADD     DX,+30h
    DEC     DI
    MOV     ES:[DI],DL
    OR      AX,AX
    JNZ     wts_cont
    MOV     AL,20h    ; ' '
    OR      SI,SI
    JNS     wts_exit
    MOV     AL,2D    ; '-'
wts_exit:    
    DEC     DI
    STOSB
    APOP    ES,SI,DI,DX,CX,BX,AX
    RETF
WORD_TO_DEC_STR ENDP
comment 
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -( CONVERT )
WORD_TO_DEC_CRT - convert word to decimal ascii and display
;
; inputs:  ax = binary value
;          di = display offset
;          ch = color
;          
; outputs: none
;
; registers changed:  ax,dx
;* * * * * * * * * * * * * *

ten	dw	10

 	public	WORD_TO_DEC_CRT
WORD_TO_DEC_CRT   	proc	far
		push	es
		mov	es,cs:lib_info.crt_seg
		call	word_to_crt1
		pop	es
		retf
WORD_TO_DEC_CRT	endp

word_to_crt1	proc	near
		xor	dx,dx
		div	cs:ten			;divide curent binary val by 10
		or	ax,ax			;check if done
		jz	db_display		;  jmp if done
		push	dx			;save remainder
		call	word_to_crt1   		;recursion
		pop	dx
db_display:	mov	al,dl
		add	al,30h
		mov	ah,ch			;get color
		stosw
		ret
word_to_crt1	endp		
;---------------------------------------------------------------------------

LIBSEG	ENDS
	end
;-----------------------------------------------------
;CONVERT WORD TO DECIMAL

;Takes a number in AX and places the ascii value at DS:SI
;AX, BX, CX, and flags are messed up

conw	proc near
	mov cx,5
	mov bx,10
	add si,4
conaaa: mov dx,0
	div bx
	mov [si],dl
	dec si
	loop conaaa
	inc si
	ret
conw	endp
>;===============================================================
>; Subroutine to convert a binary value to decimal and display it.
>;  inputs:  ax = binary value
>;  outputs: none
>;
>;  registers destroyed:  ax,dx
>;
>  
>display_binary:   xor   dx,dx
>                  div   cs:ten         ;divide value by 10 <in bx
>                  test  ax,ax          ;check if done
>                  push  dx             ;save remainder
>                  jz    db_display     ;jmp if done
>                  call  display_binary ;recursion
>db_display:       pop   ax
>                  add   al,30h
>                  int   29h
>                  ret
>  
>ten               dw 10
;------------------------------------------------------------------------
; convert binary to decimal-ascii
;  inputs AX = binary
;         DS:SI = end of string area padded with blanks
;
binary_to_ascii:
	mov	bx,10
bin_to_asc_loop:
	cmp	ax,10
	jb	last_digit		;jump if done
	xor	dx,dx			;clear dx
	div	bx			;divide number by 10
	or	dl,30h			;convert remainder to ascii
	mov	[si],dl			;store ascii char.
	dec	si			;move towards front of string
	jmp	short bin_to_asc_loop	;loop till done
last_digit:
	or	al,30h			;form last ascii char.
	mov	[si],al			;store last ascii char.
	ret
  
;==========================================================================
;  inputs:  ax = binary value
;  outputs: none
;
;  registers destroyed:  ax,dx
;
  
display_binary:	xor	dx,dx
		div	cs:ten			;divide curent binary val by 10
		or	ax,ax			;check if done
		jz	db_display		;  jmp if done
		push	dx			;save remainder
		call	display_binary		;recursion
		pop	dx
db_display:	add	dl,30h			;convert char. to ascii
		mov	ah,2
		int	21h			;display char. in -dl-
		ret

;---------------------------------------------------------------------------
  
;==========================================================================
;  inputs:  dx,ax = binary value
;  outputs: none
;
;  registers destroyed:  ax,dx
;
  
ddisplay_binary:
		push	ax
		mov	ax,dx
		xor	dx,dx
		div	cs:ten
		mov	bx,ax
		pop	ax
		div	cs:ten
		xchg	bx,dx
;
; dx,ax = quotient
;    bx = remainder

		or	ax,ax
		jnz	dd_more
		or	dx,dx
		jnz	dd_more
		mov	dx,bx
		jmp	ddb_display
dd_more:
		push	bx			;save remainder		
	
		call	ddisplay_binary		;recursion
		pop	dx
ddb_display:	add	dl,30h			;convert char. to ascii
		mov	ah,2
		int	21h			;display char. in -dl-
		ret

;---------------------------------------------------------------------------
; binary dword to decimal ascii
;   inputs dx,ax = binary
;   ds:si points to end of stuff area
;
	public	dword_to_decimal_ascii
dword_to_decimal_ascii proc  near
	mov	word ptr ds:[si-8],3030h
	mov	word ptr ds:[si-6],3030h
	mov	word ptr ds:[si-4],3030h
	mov	word ptr ds:[si-2],3030h
	xchg	bp,dx
	mov	bx,10
	mov	cl,30h
aajj:	xchg	ax,bp
	sub	dx,dx
	div	bx
	xchg	bp,ax
	div	bx
	or	dl,cl
	mov	[si],dl
	dec	si
	cmp	ax,0
	jnz	aajj
	ret
dword_to_decimal_ascii	endp
;----------------------------------------
; binary dword to decimal ascii
;   inputs dx,ax = binary
;   ds:si points to end of stuff area
;
	public	dword_to_decimal_ascii
dword_to_decimal_ascii proc  near
	xchg	bp,dx
	mov	bx,10
	mov	cl,30h
aajj:	xchg	ax,bp
	sub	dx,dx
	div	bx
	xchg	bp,ax
	div	bx
	or	dl,cl
	mov	[si],dl
	dec	si
	cmp	ax,0
	jnz	aajj
	ret
dword_to_decimal_ascii	endp
;==========================================================================
;  inputs:  dx,ax = binary value
;  outputs: none
;
;  registers destroyed:  ax,dx
;
  
ddisplay_binary:
		push	ax
		mov	ax,dx
		xor	dx,dx
		div	cs:ten
		mov	bx,ax
		pop	ax
		div	cs:ten
		xchg	bx,dx
;
; dx,ax = quotient
;    bx = remainder

		or	ax,ax
		jnz	dd_more
		or	dx,dx
		jnz	dd_more
		mov	dx,bx
		jmp	ddb_display
dd_more:
		push	bx			;save remainder		
	
		call	ddisplay_binary		;recursion
		pop	dx
ddb_display:	add	dl,30h			;convert char. to ascii
		mov	ah,2
		int	21h			;display char. in -dl-
		ret

