;----------------------------------------------------------------------
; search_block - scan next block of sequential data for string match
;  inputs:  es:bx   =   compare string start (string terminated with zero)
;           es:di   =   compare string current location
;           ds:si   =   buffer current location
;              cx   =   buffer size (remaining byte count)
;              dl   =   case flag  0=(case match)  20h=(match either case)
;           direction flag = CLD state
;
;  output:  registers di,si,cx are updated
;           if cx=0 then no match was found, check if bx=di to determine
;                   if a partial match is in process at buffer end.
;           if cx=non zero, then ds:si points at match end.  The
;                 variable match_si decremented by one = match start.
;                 use -si- and -cx- as returned to continue searching
;                 in current buffer.
;
;  Note:  If a partial match fails and the partial match was split
;         accross two buffers, then extra processing may be needed
;         by the caller to restart the compare in the first buffer.
;
;         This programs was written by Jeff Owens, but Mike Levis
;         contributed many enhancements.
;
search_block:
	push	bp
sb_top:	
	mov	ah,es:[di]	;get compare char
sb_lp1:	jcxz	sb_at_buf_end	;jmp if at end of buffer
	call	compare
	jne	sb_lp1		;jmp if no match
;
; the first character matches, save starting point and keep matching
;
	mov	bp,si
		
sb_lp2:	inc	di		;move to next compare string posn
	mov	ah,es:[di]	;get next compare stirng char
	test	ah,ah
	je	sb_total_match	;jmp if match found
	jcxz	sb_at_buf_end	;jmp if at end of buffer
	call	compare
	je	sb_lp2		;jmp if still matching	
;
; the string only partially matched.  Start looking again.
;
        add cx, si
        mov si, bp
        sub cx, bp
	mov	si,bp			;restart the search string
	mov	di,bx			;restart the compare string
	jmp	sb_top

sb_total_match:
	mov	di,bx
sb_at_buf_end:
	pop	bp
	ret
;----------------------------------------------------------------------
; compare - compare two bytes with/without case considerations
;   inputs:  ds:si = compare buffer ptr
;            es:di = compare string ptr
;            ah    = compare string char
;            dl = case flag 0(case match)  20h(match either case)
;            direction flag = CLD state
;      
;   output:  al   modified
;            ds:si = incremented buffer ptr
;            cx    = decremented
;            processor flags set  for je/jne
;
compare:
		lodsb
		dec	cx

		cmp	al, ah
		je	@@bye

		mov	dh, al
		or	dh, dl
		sub	dh, 'a'
		cmp	dh, 'z' - 'a' + 1
		jnc	@@not_alpha

		cmp	al, 'a'
		sbb	dh, dh
		or	dh, dl
		and	dh, 0E0h
		sub	al, dh
@@not_alpha:
		cmp	al, ah
@@bye:
		ret
	