								      COMMENT ~
 STRINGS.ASM -- String Manipulation Procedures
 
   From `BLUEBOOK of ASSEMBLY ROUTINES for the IBM PC & XT'
         by Christopher L. Morgan
         Copyright (C) 1984 by The Waite Group, Inc.
 
   Purpose: These routines manipulate strings.
 
   Contents:
   ---------
   COMPARE	--  Compare two strings
   LEXINSERT	--  Lexigraphically insert
   LEXSEARCH	--  Search a lexigraphically ordered list
   LOWERCASE	--  Convert to lower case
   SORTB	--  Bubble sort a string array
   STRINSERT	--  Insert one string in another
   STRSEARCH	--  Search for one string within another
   SWITCH	--  Switch two strings

 ______________________________________________________________________________
 
   Overview:
   ---------
 	LOWERCASE & UPPERCASE convert characters within a string to the other
   case.
 	  STRSEARCH & STRINSERT require two input strings.  STRSEARCH searches
   for a copy of one string in another, and STRINSERT inserts a copy of one
   string within another.
 	 LEXSEARCH & LEXINSERT require a string and a LIST of strings as input.
   LEXSEARCH searches for the proper place to insert a string in a
   lexigraphically-ordered list of strings  LEXINSERT inserts a string in the
   proper place in a lexigraphically-ordered list of strings. Lexigraphic is
   the same as alphabetic, except these routines are ordered according to their
   ASCII code.  So all uppercase letters precede any lowercase letter.
   These procedures can easily be modified so all lowercase letters are first
   converted to uppercase before lexigraphic comparison.
  	LEXINSERT calls LEXSEARCH to find the proper spot, then calls
   STRINSERT to make the insertion.
 	COMPARE, SWITCH, AND SORTB work with string arrays.  COMPARE compares
   two strings of equal length.  It lets you compare two different entries of
   the same string array. SWITCH switches two strings of the same length  e.g.,
   two different entries of the same string array.  Both these routines are
   needed by SORTB, which performs a bubble sort of a string array.  
 	These routines take advantage of the 8088 string instructions.

_____________________________ROUTINE DESCRIPTIONS______________________________

LOWERCASE -- Convert to Lower Case

  Function: This routine converts the characters in a string to lower case.

  Input: Upon entry, DS:BX points to a string.  The first two bytes in the 
    string form a 16-bit integer which specifies its length.  The remaining
    bytes contain the characters of the string.

  Output: Upon exit, all alphabetical characters of the string are lower case.

  Registers used: AX, BX, & CX are first saved and then restored.

  Segments referenced: Upon entry, the data segment must contain the string.
_______________________________________________________________________________

UPPERCASE -- Convert to Upper Case

  Function: This routine converts the characters in a string to upper case.

  Input: Upon entry, DS:BX points to a string.  The first two bytes in the 
    string form a 16-bit integer which specifies its length.  The remaining
    bytes contain the characters of the string.

  Output: Upon exit, all alphabetical characters of the string are upper case.

  Registers used: AX, BX, & CX are first saved and then restored.

  Segments referenced: Upon entry, the data segment must contain the string.
_______________________________________________________________________________

STRSEARCH -- Search for One String Within Another

  Function: This routine searches for a copy of a source string within a
    destination string.

  Input: Upon entry, DS:BX points to a source string and ES:DX points to a
    destination string.  Each string begins with a 16-bit integer that
    specifies its length.

  Output: Upon exit, AL contains a return flag (0=not found, 0FFH=found), and
    if the search was successful DX contains the location of the first byte of
    the match in the destination.

  Registers used: AX & DX are used for output; all others are unmodified.

  Segments referenced: Upon entry, the data segment must contain the source
    string and the extra segment must contain the destination string.
_______________________________________________________________________________

STRINSERT -- Insert One String in Another

  Function: This routine inserts a source string in a specified spot in a 
    destination string.

  Input: Upon entry, DS:BX points to the source string, ES:BP points to the
    destination string, and ES:DX points to the spot in the destination where
    the source is to be placed. Each string begins with a 16-bit integer that
    begins with a 16-bit integer specifying its length.

  Output: Upon exit, the destination string has the source string inserted in
    the proper location.  The length of the destination string is increased
    accordingly.

  Registers used: SI, DI, CX, & AX are first saved and then restored.

  Segments referenced: Upon entry, the data segment must contain the source
    string and the extra segment must contain the destination string.
_______________________________________________________________________________

LEXSEARCH -- Search a Lexigraphically Ordered List

  Function: This routine searches a lexigraphically (alphabetically) ordered
    list of word strings for the proper place to insert a new word.

  Input: Upon entry, DS:BX points to a source word string, ES:BP points to the
    ordered list of words. The source word string begins with a 16-bit integer
    that is its length and is followed by the bytes of the string.  The last
    byte must be a <CR> (ASCII 13). The destination list of words begins with
    a 16-bit integer specifying its length, continuing with the words which
    must consist of ASCII characters.  The words in the list are separated by
    <CR>'s (ASCII 13).

  Output: Upon exit, AL contains a return flag (0=not found, 0FFH=found). If
    the search was successful, ES:DX contains the location of the proper place
    to insert the new word. If the word was already present, then ES:DX points
    to the location of this word in the destination.

  Registers used: AX & DX are used for output; all else is unmodified.

  Segments referenced: Upon entry, the data segment must contain the source
    string and the extra segment must contain the destination word list.
_______________________________________________________________________________

LEXINSERT -- Lexigraphically Insert

  Function: This routine inserts a word string in the proper place in a
    lexigraphically (alphabetically) ordered list of word strings.  If the
    word is already present, no insertion occurs.

  Input: Upon entry, DS:BX points to a source word string and ES:BP points to 
    the ordered list of words. The source word string begins with a 16-bit 
    integer specifying its length.  The string is ASCII and must be terminated
    by a <CR> symbol (ASCII 13). The destination list of words begins with a
    16-bit integer specifying its length and continues with words consisting
    of ASCII characters. The words are separated by <CR> symbols (ASCII 13).

  Output: Upon exit, the list has the string inserted in the proper place.

  Registers used: AX is first saved and then restored.

  Segments referenced: Upon entry, the data segment and the extra segment must
    be equal and must contain the source string and the destination word list.
_______________________________________________________________________________

COMPARE -- Compare Two Strings

  Function: This routine compares two strings of the same length.

  Input: Upon entry, DS:SI points to one string (source) and DS:DI points to
    the second string (destination). Both strings have lengths given by CX.

  Output: Upon exit, the flags show the relation of the source to destination:

            L = source string is less than destination string
	    E = source string is equal to the destination string
            G = source string is greater than the destination string	

  Registers used: AX is modified.

  Segments referenced: Upon entry, the data segment contains the source string
    and the extra segment contains the destination string.
_______________________________________________________________________________

SWITCH -- Switch Two Strings

  Function: This routine switches two strings of equal length.

  Input: Upon entry, DS:SI points to one string (source) and ES:DI points to
    the second string (destination). Both strings have lengths given by CX.

  Output: Upon exit, the strings are switched.

  Registers used: AX, CX, DI, & SI are first saved and then restored

  Segments referenced: Upon entry, the data segment contains the source and the
    extra segment contains the destination string.
_______________________________________________________________________________

SORTB -- Bubble Sort a String Array

  Function: This routine sorts a string array using bubble sort.

  Input: Upon entry, DS:SI points to a string array, CX contains the number of
    entries in the array, and DX contains the size of each entry.

  Output: Upon exit, the array is alpha-up sorted. 

  Registers used: AX, CX, DI, & SI are first saved and then restored

  Segments referenced: Upon entry, the data segment and the extra segment must
    be equal and must contain the string array.

  Routines called: COMPARE, SWITCH
 _____________________________________________________________________________
									      ~
DATAS	SEGMENT	PUBLIC
	DUMMY	DB	5 DUP(?)
DATAS	ENDS 
;_____________________________________________________________________________ 
CODES	SEGMENT
	PUBLIC LOWERCASE,UPPERCASE,STRSEARCH,STRINSERT
	PUBLIC LEXSEARCH,LEXINSERT,COMPARE,SWITCH,SORTB
	ASSUME CS:CODES,DS:DATAS
;____________________________I/O ROUTINES_____________________________________
;Routine to convert string to lower case
LOWERCASE	PROC	FAR
	PUSH	BX				;Save registers
	PUSH	CX		
	PUSH	AX
;
;Get the length
	MOV	CX,[BX]				;First two bytes contain length
	INC	BX				;Point to beginning of text
	INC	BX
;
;Loop through the bytes of the string
LOWERCASE1:
	MOV	AL,[BX]				;Get the character
	CMP	AL,'A'				;Below the upper case chars?
	JB	LOWERCASE2			;If so, skip
	CMP	AL,'Z'				;Above the upper case chars?
	JA	LOWERCASE2			;If so, skip
	OR	AL,20H				;OR bit 5 into the byte
LOWERCASE2:
	MOV	[BX],AL				;Store the character
	INC	BX				;Point to next character
	LOOP	LOWERCASE1		
	POP	AX				;Restore registers
	POP	CX
	POP	BX
	RET
LOWERCASE	ENDP
;------------------------------------------------------------------------------
;Routine to convert string to uppercase
;
UPPERCASE	PROC	FAR
	PUSH	BX				;Save registers
	PUSH	CX		
	PUSH	AX
;
;Get the length
	MOV	CX,[BX]				;First two bytes contain length
	INC	BX				;Point to beginning of text
	INC	BX
;
;Loop through the bytes of the string
UPPERCASE1:
	MOV	AL,[BX]				;Get the character
	CMP	AL,'a'				;Below the lower case chars?
	JB	UPPERCASE2			;If so, skip
	CMP	AL,'z'				;Above the lower case chars?
	JA	UPPERCASE2			;If so, skip
	AND	AL,5FH				;Mask out bit number 5
UPPERCASE2:
	MOV	[BX],AL				;Store the character
	INC	BX				;Point to next character
	LOOP	UPPERCASE1		
	POP	AX				;Restore registers
	POP	CX
	POP	BX
	RET
UPPERCASE	ENDP
;------------------------------------------------------------------------------
;Routine to search for one string within another
;
STRSEARCH	PROC	FAR
	PUSH	SI				;Save registers
	PUSH	DI				
	PUSH	CX
;
;Get length of destination and point to first byte
	MOV	SI,DX				;Use source index
	LODSW					;Get the length of destination
	MOV	CX,AX				;Use the length as a count
	MOV	DX,SI				;Text begins here
STRSEARCH1:
;
;Point indices to beginning of source and destination
	MOV	SI,BX				;Load source index
	MOV	DI,DX				;Load destination index
;
;Scan for a match
	MOV	AL,[SI+2]			;Get the first character
	CLD					;Point forward
	REPNZ	SCASB				;Scan for match
	JCXZ	STRSEARCH2			;If no match, quit
;
;Got a match of first characters -- now check the entire string
	MOV	DX,DI				;Save current dest location
	DEC	DI				;Beginning of word
	LODSW					;Get source length
	XCHG	CX,AX				;Use sourc cnt & save dest cnt
	REPZ	CMPSB				;Compare two strings
	JCXZ	STRSEARCH3			;It`s a match if no more source
;
;Continue the scan
	XCHG	CX,AX				;Use destination count
	JMP	STRSEARCH1			;Back for more scanning of dest
;
;No match is possible
STRSEARCH2:
	MOV	AL,0				;Unsuccessful outcome
	JMP	STRSEARCHXIT	
;
;Found a match
STRSEARCH3:
	DEC	DX				;Point to beginning of match
	MOV	AL,0FFH				;Successful outcome
	JMP	STRSEARCHXIT
STRSEARCHXIT:
	POP	CX				;Restore registers
	POP	DI			
	POP	SI
	RET
STRSEARCH	ENDP
;------------------------------------------------------------------------------
;Routine to insert one string within another
;
STRINSERT	PROC	FAR
	PUSH	SI				;Save registers
	PUSH	DI				
	PUSH	CX
	PUSH	AX
;
;Find current end of destination string
	MOV	SI,BP				;Start of string
	ADD	SI,ES:[SI]			;Point to next to last byte
	INC	SI				;Adjust for length information
;
;Find new end of destination string and update length
	MOV	DI,SI				;Get old end of destination
	MOV	AX,[BX]				;Get length of source
	ADD	DI,DX				;New end of destination
	ADD	ES:[BP],AX			;New length of destination
;
;Move tail of destination string out of the way
	MOV	CX,SI				;SI + DX + 1 is the count
	SUB	CX,DX		
	INC	CX
	STD					;Backward direction
REP	MOVS	BYTE PTR[DI],ES:[SI]		;Move the tail	
;
;Move source string into place
	MOV	DI,DX				;Destination of move
	MOV	SI,BX				;Source of movw
	CLD					;Forward direction
	LODSW					;Length of source
	MOV	CX,AX				;The count
	REP	MOVSB				;Make the string move
STRINSERTXIT:
	POP	AX				;Restore registers	
	POP	CX
	POP	DI			
	POP	SI
	RET
STRINSERT	ENDP
;------------------------------------------------------------------------------
;Routine to search for a word in an ordered list of words
;
LEXSEARCH	PROC	FAR
;
	PUSH	SI				;Save register
	PUSH	DI
	PUSH	CX
;
;Point to beginning of list and get its length
	MOV	DI,BP				;Beginning of list
	MOV	CX,[DI]				;Get length
	INC	DI		
	INC	DI
;
;Compare source word with words in the list
LEXSEARCH1:
	MOV	DX,DI				;Save beginning of dest word
	CLD					;Forward direction
	MOV	SI,BX				;Point to beginning of source
	INC	SI
	INC	SI
;
;Compare source word with a word of the list
LEXSEARCH2:
;
;Check for end of the list
	JCXZ	LEXSEARCH5			;End of list -- insert it
;
;Set up <CR> as scan character
	MOV	AL,13				;ASCII <CR>
;
;Check for end of source word
	CMP	[SI],AL				;Source byte = <CR>?
	JE	LEXSEARCH4			;Found end of source word
;
;Check for end of destination word
	CMP	ES:[DI],AL			;Destination byte = <CR>?
	JE	LEXSEARCH3			;No match -- go to next word
;
;Compare character by character
	DEC	CX
	CMPSB					;Check for match
	JE	LEXSEARCH2			;Matched -- check next char
	JB	LEXSEARCH5			;Too high -- this is the place
;
;Scan for next <CR>
LEXSEARCH3:
	REPNZ	SCASB				;Scan until <CR>
	JMP	LEXSEARCH1			;Next word
;
;End of source word was found
LEXSEARCH4:
	CMP	[DI],AL				;Dest character = <CR>?
	JE	LEXSEARCH6			;End of destination word?
;
;Found a spot to insert the word
LEXSEARCH5:
	MOV	AL,0FFH				;Success
	JMP	LEXSEARCHXIT
;
;Word already present
LEXSEARCH6:
	MOV	AL,00H				;Already there
	JMP	LEXSEARCHXIT
LEXSEARCHXIT:
	POP	CX				;Restore registers
	POP	DI
	POP	SI
	RET
LEXSEARCH	ENDP
;------------------------------------------------------------------------------
;Routine to insert a word in an ordered list of words
;
LEXINSERT	PROC	FAR
	PUSH	AX				;Save register
	CALL	LEXSEARCH			;Search for match
	CMP	AL,0				;Already there?
	JE	LEXINSERTXIT			;Skip, if so
	CALL	STRINSERT			;Insert the new word
LEXINSERTXIT:
	POP	AX				;Restore register
	RET	
LEXINSERT	ENDP
;------------------------------------------------------------------------------
;Routine to compare two strings
;
COMPARE	PROC	FAR
	PUSH	SI				;Save registers
	PUSH	DI				
	PUSH	CX
	REPZ	CMPSB				;One compare does it !
	POP	CX				;Restore registers
	POP	DI
	POP	SI
COMPARE	ENDP	
;------------------------------------------------------------------------------
;Routine to switch two strings
;
SWITCH	PROC	FAR
	PUSH	SI				;Save registers
	PUSH	DI	
	PUSH	CX
	PUSH	AX
	CLD					;Forward direction
SWITCH1:
	MOV	AL,[DI]				;Get byte from destination
	MOVSB					;Move from source to dest
	MOV	ES:[SI-1],AL			;Put byte in source
	LOOP	SWITCH1				;Loop back for more
	POP	AX				;Restore registers
	POP	CX
	POP	DI
	POP	SI
	RET
SWITCH	ENDP
;------------------------------------------------------------------------------
;Routine to sort a string array
;
SORTB	PROC	FAR
	PUSH	SI				;Save registers
	PUSH	DI	
	PUSH	CX
	PUSH	AX
;
;Adjust count for one less than number of items
	DEC	CX				;Adjust the count
;
;Outer loop -- for SI = 1 to N-1
SORTB1:
	PUSH	CX				;Save the count
	MOV	DI,SI				;Destination points to source
;
;Inner loop -- for DI = SI+1 to N
SORTB2:
	PUSH	CX				;Save the count
	ADD	DI,DX				;Point to next destination
	MOV	CX,DX				;Entry length
	CALL	COMPARE				;Compare the strings
	JLE	SORTB3				;Skip if source <= destination
	CALL	SWITCH				;Switch if not
SORTB3:
	POP	CX				;Restore the count 
	LOOP	SORTB2
	ADD	SI,DX				;Point to next source
	POP	CX				;Restore the count
	LOOP	SORTB1
SORTBXIT:
	POP	AX				;Restore registers
	POP	CX
	POP	DI
	POP	SI
	RET
SORTB	ENDP
;-----------------------------------------------------------------------------
CODES	ENDS
;
	END
;_____________________________________________________________________________ 
;>>>>> Physical EOF STRINGS.ASM <<<<<
