;---------------------------------------------------------------------------
  In file under CLOCK is source for:
    SCANTIME - convert time to binary
    SCANDATE - convert date to binary
  pc-mag june 14, 1988
;---------------------------------------------------------------------------
; make ascii year
;   inputs  ax=binary year 1980-2079
;   output  al=year 1      ah=year 2   
;           dl=year 3      dh=year 4
;           carry set if error found
;
	public	make_ascii_year
make_ascii_year:
	cmp	ax,400
	sub	dx,dx
	jl	low_year
	mov	cx,100
	cmp	dx,cx
	jnb	make_year_err		;jump if divide err
	div	cx
	xchg	ax,dx
	call	binary_to_ascii		;get low characters
	xchg	ax,dx
	call	binary_to_ascii		;get high characters
	ret
low_year:
	call	binary_to_ascii
	mov	dx,ax
	mov	ax,'00'
	clc
	ret
make_year_err:
	stc
	ret

	page	,132
;---------------------------------- CAL -----------------------------------
;
; build calendar header line
;
cal_build:
	mov	al,cal_month
	mov	ah,9
	mul	ah			;index into months
	add	ax,offset months -9
	mov	si,ax
	mov	di,(offset cal_outline) + 13
	mov	cx,9
month_move:
	cld
	rep	movsb
	mov	ax,cal_year
	call	make_ascii_year
	mov	word ptr ds:cal_outline+48,ax
	mov	word ptr ds:cal_outline+50,dx	
	
	mov	di,offset cal_outline		;initialize display point
	add	di,(3*71)+3			;point at first day
	mov	dh,cal_month
	mov	cx,cal_year
	SUB	CX,1900
	MOV	DL,1
	CALL	COMPUTE_DAY_OF_WEEK		;get day in -al-
	MOV	DL,AL
	MOV	CL,AL
	mov	first_day_number,al		;save for today's highlight
	SHL	CL,1
	SHL	CL,1
;
; display spaces until first day of month (cl = count)
;
	jcxz	build_cal_day_numbers
	MOV	AL,' '
S_LOOP:	mov	[di],al		;fill display buf. start with blanks
	inc	di
	DEC	CL
	JNZ	S_LOOP
build_cal_day_numbers:
	MOV	CX,cal_year
	MOV	DH,cal_month
	CALL	GET_DAYS_PER_MONTH
	MOV	CL,AL
	MOV	DH,1		;initialize day loop counter
L0285:	MOV	AL,DH		;move current day count to -al-
	AAM			;  and
	ADD	AX,'00'		;    convert to ascii
	CMP	AH,'0'		;check if first char. is 0       
	JNZ	L0293		;jump if not zero
	MOV	AH,' '		;substitute space for leading zero
L0293:	XCHG	AL,AH		;display day count (body of calendar)
	mov	[di],al
	inc	di
	XCHG	AL,AH
	mov	[di],al
	inc	di
	INC	DH		;bump day count
	CMP	DH,CL		;check if done
	JA	blanks_at_end	;jump if all done
	INC	DL		;bump days per line
	CMP	DL,7		;check for end of line
	JZ	END_OF_LINE		;jump if at end of line
	MOV	AX,'  '
	mov	[di],ax
	add	di,2
	JMP	SHORT	L0285

END_OF_LINE:
	add	di,45		;wrap count
	MOV	DL,0
	JMP	SHORT	L0285
	
next_cal_line:
	add	di,43
blanks_at_end:
	mov	byte ptr [di],' '
	inc	di
	cmp	di,(offset cal_outline) + (6*71) + 30
	je	next_cal_line
	cmp	di,(offset cal_outline) + (7*71) + 30
	je	next_cal_line		;jump if at end of line
	cmp	di,(offset cal_outline) + (8*71) + 30
	je	next_cal_line		;jump if at end of line
	cmp	di,(offset cal_outline) + (9*71) + 30
	jle	blanks_at_end
;
	mov	si,offset cal_outline
	mov	dx,0304h			;start on line 4 column 4
	mov	cx,70				;line size
	call	pop_display

	mov	ah,highlight_attribute		;highlight display
	mov	bx,offset cal_highlight_table
	call	highlight
;
; check if todays date appears on the calendar
;
	mov	ax,cal_year
	cmp	ax,todays_binary_year
	jne	keyboard_input			;jump if not current year
	mov	al,cal_month
	cmp	al,todays_binary_month
	jne	keyboard_input			;jump if not current month
;
; highlight todays date on displayed calendar
;
	mov	dx,0607h			;get starting positon for days
	mov	al,todays_binary_day		;get column #
	dec	al				;  convert to column 0-?
	add	al,first_day_number		;  add todays date
cal_lp:	cmp	al,7				;check if day on current line
	jb	fix_col				;  and jump if it is
	sub	al,7				;move to next line
	add	dh,1				;move to next line
	jmp	cal_lp				;loop till correct line found
fix_col:mov	ah,4				;get space for each day
	mul	ah				;increment into line
	add	dl,al				;compute display column
	mov	todays_highlight,dx		;save highlighted date address
	
	mov	cx,2				;highlight 2 columns
	mov	bl,highlight_attribute
	call	write_attribute			;highlight todays date
keyboard_input:
	mov	bx,offset cal_decode_keys
	call	decode_key

	push	bx
	mov	ah,normal_attribute		;remove highlights
	mov	bx,offset cal_highlight_table
	call	highlight
	mov	dx,todays_highlight		;remove highlight
	mov	cx,2
	mov	bl,normal_attribute		;  of todays
	call	write_attribute			;    date
	pop	bx

	call	bx				;go respond to key press
	jmp	cal_build
cal_exit:
	pop	ds				;clear stack return adr
	pop	ds           			;restore seg registers
	ret
;---------------------------------------
cal_month_fwd:
	mov	ch,cal_month
	mov	dx,cal_year
	inc	ch
	cmp	ch,13			;check for end of year
	jl	month2			;jump if month is ok
	mov	ch,1			;move month to january
	inc	dx			;move to next year
month2:	mov	cal_month,ch
	mov	cal_year,dx
	jmp	short new_calendar
;---------------------------------------
cal_month_bak:
	mov	ch,cal_month
	mov	dx,cal_year
	dec	ch
	jnz	month3			;jump if month ok
	mov	ch,12
	dec	dx
month3:	mov	cal_month,ch
	mov	cal_year,dx
	jmp	short new_calendar
;-----------------------------------
cal_year_fwd:
	inc	cal_year
	jmp	short new_calendar
;------------------------------------
cal_year_bak:
	dec	cal_year
new_calendar:
	ret
;-------------------------------
;
COMPUTE_DAY_OF_WEEK:
	MOV	AL,CL		;get (year - 1900) in -al-
	DEC	AL
	XOR	AH,AH
	PUSH	DX
	MOV	BX,365		;multiply (year - 1901) * 365
	MUL	BX
	POP	DX
	MOV	BL,CL		;divide
	DEC	BL		;  (year - 1901)
	SHR	BL,1		;     by
	SHR	BL,1		;       4
	XOR	BH,BH
	ADD	AX,BX		;add (year-1901)*365 + (year-1901)/4
	XOR	BX,BX
;
; add in the days in current year, starting with months
;
L032E:	INC	BL
	CMP	BL,DH		;check if count = current month
	JZ	L0348		;exit if all months added
	ADD	AL,[BX+DAYS_for_MONTH]
	ADC	AH,0
	CMP	BL,2		;check for february?
	JNZ	L032E
	TEST	CL,3		;check year for 1903?
	JNZ	L032E
	INC	AX		;adjust for leap year?
	JMP	SHORT	L032E
L0348:	ADD	AL,DL
	ADC	AH,0
;
; take mod7 of -ax- to get day of week
;
	XOR	DX,DX
	INC	AX
	DIV	WORD PTR SEVEN
	MOV	AL,DL
	ret
;----------------------------------------------------------------------------
;
GET_DAYS_PER_MONTH:
	MOV	BL,DH
	XOR	BH,BH
	MOV	AL,[BX+DAYS_for_MONTH]
	CMP	DH,2
	JNZ	L0370
	TEST	CL,3
	JNZ	L0370
	INC	AL
L0370:	ret
;--------------------------
	
_TEXT	ENDS

	end
	
;----------------------------------------------------------------------;
; Input:                                                               ;
;   CX = Year  (1980 - 2099)                                           ;
;   DH = Month (1 - 12)                                                ;
;   DL = Day   (1 - 31)                                                ;
;                                                                      ;
; Output:                                                              ;
;   DX = compressed date in directory entry format.                    ;
;                                                                      ;
;        <     DH      > <     DL      >                               ;
;        y y y y y y y m m m m d d d d d                               ;
;                                                                      ;
;              y = year  (0 - 119)                                     ;
;              m = month (1 - 12)                                      ;
;              d = day   (1 - 31)                                      ;
;                                                                      ;
;   BX, CX destroyed.                                                  ;
;----------------------------------------------------------------------;

convert_date    proc near

                sub cx,80
                cmp cx,1900
                jb save_month
                sub cx,1900
save_month:     mov bl,dh
                xor bh,bh
                shl cl,1
                mov dh,cl
                mov cl,5
                shl bx,cl
                or dx,bx
                ret

convert_date    endp

;--------------------------------------------------------------;
; Input:                                                       ;
;                                                              ;
;                                                              ;
;                                                              ;
;                                                              ;
; Output:                                                      ;
;                                                              ;
;                                                              ;
;                                                              ;
;                                                              ;
;                                                              ;
;                                                              ;
;                                                              ;
;                                                              ;
;                                                              ;
;                                                              ;
;--------------------------------------------------------------;

convert_time    proc near

                mov bx,cx
                mov cl,9
                shr dx,cl
                mov cl,3
                shl bh,cl
                mov dh,bh
                mov cl,5
                shl bx,cl
                or dx,bx
                ret
convert_time    endp
